unit untdefs;

interface

uses Windows, SysUtils, PSApi;

type
 NTSTATUS=cardinal;
 POBJECT=pointer;

 USHORT=word;

 SYSTEM_HANDLE=packed record
	uIdProcess:ULONG;
	ObjectType:UCHAR;    // OB_TYPE_* (OB_TYPE_TYPE, etc.)
	Flags:UCHAR;         // HANDLE_FLAG_* (HANDLE_FLAG_INHERIT, etc.)
	Handle:USHORT;
	pObject:PObject;
	GrantedAccess:ACCESS_MASK;
 end;
 PSYSTEM_HANDLE=^SYSTEM_HANDLE;
 SYSTEM_HANDLE_ARRAY=array[0..0] of SYSTEM_HANDLE;
 PSYSTEM_HANDLE_ARRAY=^SYSTEM_HANDLE_ARRAY;

 SYSTEM_HANDLE_INFORMATION=packed record
	uCount:ULONG;
	Handles:SYSTEM_HANDLE_ARRAY;
 end;
 PSYSTEM_HANDLE_INFORMATION=^SYSTEM_HANDLE_INFORMATION;

 UNICODE_STRING=packed record
    Length:USHORT;
    MaximumLength:USHORT;
    Buffer:PWideChar;
 end;
 PUNICODE_STRING=^UNICODE_STRING;

 OBJECT_NAME_INFORMATION=UNICODE_STRING;
 POBJECT_NAME_INFORMATION=^OBJECT_NAME_INFORMATION;

 SYSTEM_INFORMATION_CLASS=(
  SystemBasicInformation,       // 0x002C
  SystemProcessorInformation,     // 0x000C
  SystemPerformanceInformation,    // 0x0138
  SystemTimeInformation,       // 0x0020
  SystemPathInformation,       // not implemented
  SystemProcessInformation,      // 0x00F8+ per process
  SystemCallInformation,       // 0x0018 + (n * 0x0004)
  SystemConfigurationInformation,   // 0x0018
  SystemProcessorCounters,      // 0x0030 per cpu
  SystemGlobalFlag,          // 0x0004
  SystemInfo10,            // not implemented
  SystemModuleInformation,      // 0x0004 + (n * 0x011C)
  SystemLockInformation,       // 0x0004 + (n * 0x0024)
  SystemInfo13,            // not implemented
  SystemPagedPoolInformation,     // checked build only
  SystemNonPagedPoolInformation,   // checked build only
  SystemHandleInformation,      // 0x0004 + (n * 0x0010)
  SystemObjectInformation,      // 0x0038+ + (n * 0x0030+)
  SystemPagefileInformation,     // 0x0018+ per page file
  SystemInstemulInformation,     // 0x0088
  SystemInfo20,            // invalid info class
  SystemCacheInformation,       // 0x0024
  SystemPoolTagInformation,      // 0x0004 + (n * 0x001C)
  SystemProcessorStatistics,     // 0x0000, or 0x0018 per cpu
  SystemDpcInformation,        // 0x0014
  SystemMemoryUsageInformation1,   // checked build only
  SystemLoadImage,          // 0x0018, set mode only
  SystemUnloadImage,         // 0x0004, set mode only
  SystemTimeAdjustmentInformation,  // 0x000C, 0x0008 writeable
  SystemMemoryUsageInformation2,   // checked build only
  SystemInfo30,            // checked build only
  SystemInfo31,            // checked build only
  SystemCrashDumpInformation,     // 0x0004
  SystemExceptionInformation,     // 0x0010
  SystemCrashDumpStateInformation,  // 0x0008
  SystemDebuggerInformation,     // 0x0002
  SystemThreadSwitchInformation,   // 0x0030
  SystemRegistryQuotaInformation,   // 0x000C
  SystemLoadDriver,          // 0x0008, set mode only
  SystemPrioritySeparationInformation,// 0x0004, set mode only
  SystemInfo40,            // not implemented
  SystemInfo41,            // not implemented
  SystemInfo42,            // invalid info class
  SystemInfo43,            // invalid info class
  SystemTimeZoneInformation,     // 0x00AC
  SystemLookasideInformation,     // n * 0x0020
// info classes specific to Windows 2000
// WTS = Windows Terminal Server
  SystemSetTimeSlipEvent,       // set mode only
  SystemCreateSession,        // WTS, set mode only
  SystemDeleteSession,        // WTS, set mode only
  SystemInfo49,            // invalid info class
  SystemRangeStartInformation,    // 0x0004
  SystemVerifierInformation,     // 0x0068
  SystemAddVerifier,         // set mode only
  SystemSessionProcessesInformation // WTS
  );

 OBJECT_INFORMATION_CLASS = (
	ObjectBasicInformation,
	ObjectNameInformation,
	ObjectTypeInformation,
	ObjectAllTypesInformation,
	ObjectHandleInformation
 );

 TNTQuerySystemInformation=function
  (SystemInformationClass:DWORD;
   SystemInformation:pointer;
   SystemInformationLength:DWORD;
   ReturnLength:PDWORD
   ):NTSTATUS; stdcall;

 TNTQueryObject=function(
  ObjectHandle:cardinal;
  ObjectInformationClass:OBJECT_INFORMATION_CLASS;
  ObjectInformation:pointer;
  Length:ULONG;
  ResultLength:PDWORD
  ):NTSTATUS;stdcall;  

const
 STATUS_SUCCESS:NTSTATUS	= $00000000;
 STATUS_INFO_LENGTH_MISMATCH:NTSTATUS = $C0000004;
 STATUS_BUFFER_OVERFLOW:NTSTATUS	= $80000005;

function EnableDebugPrivilege:boolean;
function GetObjectInfo(hObject:cardinal;
 objInfoClass:OBJECT_INFORMATION_CLASS):LPWSTR;
function  GetFileNameFromHandle(hFile:Cardinal):string;

var
 NTQueryObject:TNTQueryObject;
 NTQuerySystemInformation:TNTQuerySystemInformation;


implementation

function EnableDebugPrivilege:boolean;
var
	hToken:cardinal;
	tokenPriv:TOKEN_PRIVILEGES;
	luidDebug:TLargeInteger;
  rlen:cardinal;
begin
  result:=false;
	if OpenProcessToken(GetCurrentProcess(),
      TOKEN_ADJUST_PRIVILEGES,
      hToken) then begin
		if LookupPrivilegeValue( nil , 'SeDebugPrivilege', luidDebug) then
		begin
			tokenPriv.PrivilegeCount := 1;
			tokenPriv.Privileges[0].Luid := luidDebug;
			tokenPriv.Privileges[0].Attributes := SE_PRIVILEGE_ENABLED;
			result:=AdjustTokenPrivileges(hToken, FALSE, tokenPriv,
        sizeof(tokenPriv), nil, rlen);
		end;
	end;
end;

function GetObjectInfo(hObject:cardinal;
 objInfoClass:OBJECT_INFORMATION_CLASS):LPWSTR;
var
 dwSize:DWORD;
 pObjectInfo:POBJECT_NAME_INFORMATION;
 ntReturn:NTSTATUS;
begin
  result:=nil;
	if(@NTQuerySystemInformation<>nil) then begin
		dwSize := sizeof(OBJECT_NAME_INFORMATION);
		pObjectInfo := AllocMem(dwSize);
		ntReturn := NTQueryObject(hObject, objInfoClass, pObjectInfo,
     dwSize, @dwSize);
		if((ntReturn = STATUS_BUFFER_OVERFLOW) or
       (ntReturn = STATUS_INFO_LENGTH_MISMATCH)) then begin
			FreeMem(pObjectInfo);
			pObjectInfo := AllocMem(dwSize);
			ntReturn := NTQueryObject(hObject, objInfoClass, pObjectInfo,
       dwSize, @dwSize);
		end;
		if((ntReturn >= STATUS_SUCCESS) and (pObjectInfo.Buffer <> nil)) then
		begin
			result := AllocMem(pObjectInfo.Length + sizeof(WCHAR));
			CopyMemory(result, pObjectInfo.Buffer, pObjectInfo.Length);
		end;
		FreeMem(pObjectInfo);
	end;
end;

function  GetFileNameFromHandle(hFile:Cardinal):string;
const
 BUFSIZE=512;
var
 bFound:boolean;
 pszFilename,
 szTemp,
 szDrive,
 szName:PAnsiChar;
 p:PAnsiChar;
 hFileMap,uNameLen:Cardinal;
 dwFileSizeHi, dwFileSizeLo:DWORD;
 pMem:pointer;
begin
   result:='';
   //  Get  the  file  size.
   dwFileSizeHi  :=  0;
   dwFileSizeLo  :=  GetFileSize(hFile,  @dwFileSizeHi);

   if ((dwFileSizeLo  =  0)  and  (dwFileSizeHi  =  0)) then exit;

   pszFilename:=AllocMem(MAX_PATH*2+2);
   szTemp:=AllocMem(BUFSIZE);
   szName:=AllocMem(MAX_PATH);

   //  Create  a  file  mapping  object.
   hFileMap  :=  CreateFileMapping(hFile,
                     nil,
                     PAGE_READONLY,
                     0,
                     1,
                     nil);

    if  (hFileMap<>0) then begin
    //  Create  a  file  mapping  to  get  the  file  name.
    pMem  :=  MapViewOfFile(hFileMap,  FILE_MAP_READ,  0,  0,  1);
     if  (pMem<>nil) then begin
       if  GetMappedFileName (GetCurrentProcess(),  
                              pMem,  
                              pszFilename, 
                              MAX_PATH)<>0 then
       begin
          //  Translate  path  with  device  name  to  drive  letters. 
          if  (GetLogicalDriveStrings(BUFSIZE-1,  szTemp)<>0) then begin
           bFound  :=  FALSE;
           p  :=  szTemp;
           repeat
             //  Copy  the  drive  letter  to  the  template  string
             szDrive  :=  p; 
             //  Look  up  each  device  name
             if  QueryDosDevice(szDrive,  szName,  BUFSIZE)<>0 then begin
              uNameLen  :=  length(szName);
              if  (uNameLen  <  MAX_PATH) then begin
                bFound  := AnsiStrLComp(pszFilename, szName,uNameLen)=0;

                if  (bFound) then begin
                   //  Reconstruct  pszFilename  using  szTemp
                   //  Replace  device  path  with  DOS  path
                   result:=Format('%s%s',
                             [szDrive,
                             PAnsiChar(cardinal(pszFilename)+
                             uNameLen)]);
                end;
              end;
             end;

            //  Go  to  the  next  NULL  character.
            while ( p^<>#0 ) do inc(p);
           until bFound or (p^=#0);
         end;
       end;
       UnmapViewOfFile(pMem); 
     end;  

      CloseHandle(hFileMap); 
   end;

   FreeMem(pszFilename);
   FreeMem(szTemp);
   FreeMem(szName);
end;

begin
 NTQueryObject := GetProcAddress(
  GetModuleHandle('NTDLL.DLL'), 'NtQueryObject');
 NTQuerySystemInformation := GetProcAddress(
  GetModuleHandle('NTDLL.DLL'), 'NtQuerySystemInformation');
end.
